''********************************
''* KEYBOARD_EN.spin 1.1  3/2016 *
''* Author: Werner L. Schneider  *
''********************************

'' Based on PS/2 Keyboard Driver v1.0.1 by Chip Gracey
'' Copyright (c) 2004 Parallax, Inc.
''
'' Changes in v1.1
''
'' Ctrl + Alt + Del Propeller Restart
''
'' 
VAR

  long  cog

  long  par_tail        'key buffer tail        read/write      (12 contiguous longs)
  long  par_head        'key buffer head        read-only
  long  par_present     'keyboard present       read-only
  long  par_keys[8]     'key buffer (16 words)  read-only       (also used to pass initial parameters)


PUB start(dpin, cpin) : okay

'' Start keyboard driver - starts a cog
'' returns false if no cog available
''
''   dpin  = data signal on PS/2 jack
''   cpin  = clock signal on PS/2 jack
''
''     use 100-ohm resistors between pins and jack
''     use 10K-ohm resistors to pull jack-side signals to VDD
''     connect jack-power to 5V, jack-gnd to VSS
''
'' NumLock will be initially 'on'
'' and auto-repeat will be set to 15cps with a delay of .5s

  okay := startx(dpin, cpin, 1, %01_01000)


PUB startx(dpin, cpin, numlock, auto) : okay

'' Like start, but allows you to specify lock settings and auto-repeat
''
''   numlock = numlock setup
''             1 = on
''
''   auto  = auto-repeat setup
''           bits 6..5 specify delay (0=.25s, 1=.5s, 2=.75s, 3=1s)
''           bits 4..0 specify repeat rate (0=30cps..31=2cps)
''           (eg %01_00000 = .5s delay, 30cps repeat)

  stop
  longmove(@par_keys, @dpin, 4)
  okay := cog := cognew(@entry, @par_tail) + 1


PUB stop

'' Stop keyboard driver - frees a cog

  if cog
    cogstop(cog~ -  1)
  longfill(@par_tail, 0, 11)


PUB present : truefalse

'' Check if keyboard present - valid ~2s after start
'' returns t|f

  truefalse := -par_present


PUB key : keycode

'' Get key (never waits)
'' returns key (0 if buffer empty)

  if par_tail <> par_head
    keycode := par_keys.word[par_tail]
    par_tail := ++par_tail & $F


DAT

'******************************************
'* Assembly language PS/2 keyboard driver *
'******************************************

                        org
'
'
' Entry
'
entry                   movd    :par,#_dpin             'load input parameters _dpin/_cpin/_locks/_auto
                        mov     x,par
                        add     x,#3*4
                        mov     y,#4
:par                    rdlong  0,x
                        add     :par,dlsb
                        add     x,#4
                        djnz    y,#:par

                        mov     dmask,#1                'set pin masks
                        shl     dmask,_dpin
                        mov     cmask,#1
                        shl     cmask,_cpin

                        test    _dpin,#$20      wc      'modify port registers within code
                        muxc    _d1,dlsb
                        muxc    _d2,dlsb
                        muxc    _d3,#1
                        muxc    _d4,#1
                        test    _cpin,#$20      wc
                        muxc    _c1,dlsb
                        muxc    _c2,dlsb
                        muxc    _c3,#1

                        mov     _head,#0                'reset output parameter _head
'
'
' Reset keyboard
'
reset                   mov     dira,#0                 'reset directions
                        mov     dirb,#0

                        mov     _present,#0
                        mov     sca,#0
                        mov     shiftlock,#0
                        
                        mov     stat,#8                 'set reset flag
'
'
' Update parameters
'
update                  movd    :par,#_head             'update output parameters _head/_present/_ctrlaltdel
                        mov     x,par
                        add     x,#1*4
                        mov     y,#1+1
:par                    wrlong  0,x
                        add     :par,dlsb
                        add     x,#4
                        djnz    y,#:par

                        test    stat,#8         wc      'if reset flag, transmit reset command
        if_c            mov     data,#$FF
        if_c            call    #transmit
'
'
' Get scancode
'
newcode                 mov     stat,#0                 'reset state

:same                   call    #receive                'receive byte from keyboard

                        cmp     data,#$84+1     wc      'scancode?

        if_nc           cmp     data,#$AA       wz      'powerup/reset?
        if_nc_and_z     jmp     #configure

        if_nc           cmp     data,#$F0       wz      'released?
        if_nc_and_z     or      stat,#2
        if_nc_and_z     jmp     #:same

        if_nc           jmp     #newcode                'unknown, ignore



'Set Shift/Ctrl/Alt

                        cmp     data,#$59       wz      'Test Shift-R
        if_z            mov     data,#$12               
                        cmp     data,#$58       wz      'Test Ctrl-R
        if_z            mov     data,#$11               
                        cmp     data,#$39       wz      'Test Alt-R
        if_z            mov     data,#$19               

                        cmp     data,#$12       wz      'Test Shift-L
        if_z            test    stat,#2         wc                
        if_c_and_z      and     sca,#%1110
        if_nc_and_z     or      sca,#%0001

                        cmp     data,#$11       wz      'Test Ctrl-L
        if_z            test    stat,#2         wc                
        if_c_and_z      and     sca,#%1101
        if_nc_and_z     or      sca,#%0010

                        cmp     data,#$19       wz      'Test Alt-L
        if_z            test    stat,#2         wc                
        if_c_and_z      and     sca,#%1011
        if_nc_and_z     or      sca,#%0100

                        test    stat,#2         wc
        if_nc           cmpsub  data,#$F0       wc      
        if_c            jmp     #update

                        cmp     data,#$14       wz      'Set/Reset ShiftLock
        if_z            xor     shiftlock,#1
        if_z            jmp     #configure

                        cmp     data,#$76       wz      'Set/Reset NumLock
        if_z            xor     _numlock,#1
        if_z            jmp     #configure

                        cmp     shiftlock,#1   wz                         
        if_nz           jmp     #tstsca
                        test    shiftlock,#1   wc      
                        rcl     data,#1
                        call    #look
                        jmp     #nosca

tstsca                  test    sca,#%001       wc      
                        rcl     data,#1
                        call    #look

nosca                   cmpsub  data,#$D0       wc      'translate keypad, considering numlock
        if_c            test    _numlock,#1     wz
        if_c_and_z      add     data,#@keypad1-@table
        if_c_and_nz     add     data,#@keypad2-@table
        if_c            call    #look


chkctrlaltdel
                        cmp     sca,#%0110      wz
        if_z            cmp     data,#$B9       wz
        if_z            mov     data,#$ED               'set keyboard Led's
        if_z            call    #transmit
        if_z            mov     data,#%000              'all off
        if_z            call    #transmit
        if_z            mov     x,#%00000000
        if_z            mov     y,#0 
        if_z            mov     t,all
waitr   if_z            wrbyte  x,y
        if_z            add     y,#1
        if_z            djnz    t,#waitr        
        if_z            clkset  %10000000               'restart            

done                    rdlong  x,par                   'if room in buffer and key valid, enter
                        sub     x,#1
                        and     x,#$F
                        cmp     x,_head         wz
        if_nz           test    data,#$FF       wz
        if_nz           mov     x,par
        if_nz           add     x,#3*4
        if_nz           add     x,_head
        if_nz           add     x,_head
        if_nz           wrword  data,x
        if_nz           add     _head,#1
        if_nz           and     _head,#$F

                        test    stat,#4         wc      'if not configure flag, done
        if_nc           jmp     #update                 'else configure to update leds
'
'
' Configure keyboard
'
configure               mov     data,#$F0               'set keyboard scancode 3
                        call    #transmit
                        mov     data,#$03
                        call    #transmit

                        mov     data,#$FA               'set all Keys Make/Break/Typematic
                        call    #transmit

                        mov     data,#$F3               'set keyboard auto-repeat
                        call    #transmit
                        mov     data,_auto
                        and     data,#%00_00000
                        call    #transmit

                        mov     data,#$ED               'set keyboard lock-leds
                        call    #transmit
                        mov     data,#0
                        mov     x,_numlock
                        test    x,#1            wc                         
        if_c            or      data,#%010
                        mov     x,shiftlock
                        test    x,#1            wc                         
        if_c            or      data,#%100
                        call    #transmit

                        mov     _present,#1             'set _present

                        jmp     #update                 'done
'
'
' Lookup byte in table
'
look                    ror     data,#2                 'perform lookup
                        movs    :reg,data
                        add     :reg,#table
                        shr     data,#27
                        mov     x,data
:reg                    mov     data,0
                        shr     data,x

                        jmp     #rand                   'isolate byte
'
'
' Transmit byte to keyboard
'
transmit
_c1                     or      dira,cmask              'pull clock low
                        movs    napshr,#13              'hold clock for ~128us (must be >100us)
                        call    #nap
_d1                     or      dira,dmask              'pull data low
                        movs    napshr,#18              'hold data for ~4us
                        call    #nap
_c2                     xor     dira,cmask              'release clock

                        test    data,#$0FF      wc      'append parity and stop bits to byte
                        muxnc   data,#$100
                        or      data,dlsb

                        mov     x,#10                   'ready 10 bits
transmit_bit            call    #wait_c0                'wait until clock low
                        shr     data,#1         wc      'output data bit
_d2                     muxnc   dira,dmask
                        mov     wcond,c1                'wait until clock high
                        call    #wait
                        djnz    x,#transmit_bit         'another bit?

                        mov     wcond,c0d0              'wait until clock and data low
                        call    #wait
                        mov     wcond,c1d1              'wait until clock and data high
                        call    #wait

                        call    #receive_ack            'receive ack byte with timed wait
                        cmp     data,#$FA       wz      'if ack error, reset keyboard
        if_nz           jmp     #reset

transmit_ret            ret
'
'
' Receive byte from keyboard
'
receive                 test    _cpin,#$20      wc      'wait indefinitely for initial clock low
                        waitpne cmask,cmask
receive_ack
                        mov     x,#11                   'ready 11 bits
receive_bit             call    #wait_c0                'wait until clock low
                        movs    napshr,#16              'pause ~16us
                        call    #nap
_d3                     test    dmask,ina       wc      'input data bit
                        rcr     data,#1
                        mov     wcond,c1                'wait until clock high
                        call    #wait
                        djnz    x,#receive_bit          'another bit?

                        shr     data,#22                'align byte
                        test    data,#$1FF      wc      'if parity error, reset keyboard
        if_nc           jmp     #reset
rand                    and     data,#$FF               'isolate byte

look_ret
receive_ack_ret
receive_ret             ret
'
'
' Wait for clock/data to be in required state(s)
'
wait_c0                 mov     wcond,c0                '(wait until clock low)

wait                    mov     y,tenms                 'set timeout to 10ms

wloop                   movs    napshr,#18              'nap ~4us
                        call    #nap
_c3                     test    cmask,ina       wc      'check required state(s)
_d4                     test    dmask,ina       wz      'loop until got state(s) or timeout
wcond   if_never        djnz    y,#wloop                '(replaced with c0/c1/c0d0/c1d1)

                        tjz     y,#reset                'if timeout, reset keyboard
wait_ret
wait_c0_ret             ret


c0      if_c            djnz    y,#wloop                '(if_never replacements)
c1      if_nc           djnz    y,#wloop
c0d0    if_c_or_nz      djnz    y,#wloop
c1d1    if_nc_or_z      djnz    y,#wloop
'
'
' Nap
'
nap                     rdlong  t,#0                    'get clkfreq
napshr                  shr     t,#18/16/13             'shr scales time
                        min     t,#3                    'ensure waitcnt won't snag
                        add     t,cnt                   'add cnt to time
                        waitcnt t,#0                    'wait until time elapses (nap)

nap_ret                 ret
'
'
' Initialized data
'
'
dlsb                    long    1 << 9
tenms                   long    10_000 / 4
all                     long    32768 
'
'
' Lookup table
'                               ascii   scan    extkey  regkey  ()=keypad
'

table                   word    $0000    '00
                        word    $0000    '01
                        word    $0000    '02
                        word    $0000    '03
                        word    $0000    '04
                        word    $0000    '05
                        word    $0000    '06
                        word    $C0C0    '07    F1
                        word    $1B1B    '08    Esc
                        word    $0000    '09
                        word    $0000    '0A
                        word    $0000    '0B
                        word    $0000    '0C
                        word    $0909    '0D    Tab
                        word    $7E60    '0E    ~ `
                        word    $C1C1    '0F    F2
                        word    $0000    '10
                        word    $0000    '11    Ctrl-L 
                        word    $0000    '12    Shift-L
                        word    $0000    '13    
                        word    $0000    '14    Capslock
                        word    $5171    '15    Q q
                        word    $2131    '16    ! 1
                        word    $C2C2    '17    F3
                        word    $0000    '18
                        word    $0000    '19    Alt-L
                        word    $5A7A    '1A    Z z
                        word    $5373    '1B    S s
                        word    $4161    '1C    A a
                        word    $5777    '1D    W w
                        word    $4032    '1E    @ 2
                        word    $C3C3    '1F    F4
                        word    $0000    '20
                        word    $4363    '21    C c
                        word    $5878    '22    X x
                        word    $4464    '23    D d
                        word    $4565    '24    E e
                        word    $2434    '25    $ 4
                        word    $2333    '26    # 3
                        word    $C4C4    '27    F5
                        word    $0000    '28
                        word    $2020    '29    Space
                        word    $5676    '2A    V v
                        word    $4666    '2B    F f
                        word    $5474    '2C    T t
                        word    $5272    '2D    R r
                        word    $2535    '2E    % 5
                        word    $C5C5    '2F    F6
                        word    $0000    '30
                        word    $4E6E    '31    N n
                        word    $4262    '32    B b
                        word    $4868    '33    H h
                        word    $4767    '34    G g
                        word    $5979    '35    Y y
                        word    $5E36    '36    ^ 6
                        word    $C6C6    '37    F7
                        word    $0000    '38
                        word    $0000    '39    Alt-R 
                        word    $4D6D    '3A    M m
                        word    $4A6A    '3B    J j
                        word    $5575    '3C    U u
                        word    $2637    '3D    & 7
                        word    $2A38    '3E    * 8
                        word    $C7C7    '3F    F8
                        word    $0000    '40
                        word    $3C2C    '41    < ,
                        word    $4B6B    '42    K k
                        word    $4969    '43    I i
                        word    $4F6F    '44    O o
                        word    $2930    '45    ) 0 
                        word    $2839    '46    ( 9
                        word    $C8C8    '47    F9
                        word    $0000    '48
                        word    $3E2E    '49    > .
                        word    $3F2F    '4A    ? /
                        word    $4C6C    '4B    L l
                        word    $3A3B    '4C    : ;
                        word    $5070    '4D    P p
                        word    $5F2D    '4E    _ -
                        word    $C9C9    '4F    F10
                        word    $0000    '50
                        word    $0000    '51
                        word    $2227    '52    " '
                        word    $7C5C    '53    | \
                        word    $7B5B    '54    { [    
                        word    $2B3D    '55    + =
                        word    $CACA    '56    F11
                        word    $CCCC    '57    Print Screen
                        word    $0000    '58    Ctrl-R 
                        word    $0000    '59    Shift-R
                        word    $0D0D    '5A    Enter
                        word    $7D5D    '5B    } ]
                        word    $0000    '5C
                        word    $0000    '5D
                        word    $CBCB    '5E    F12
                        word    $0000    '5F    Scrolllock
                        word    $B3B3    '60    Down  
                        word    $B0B0    '61    Left  
                        word    $0000    '62
                        word    $B2B2    '63    Up    
                        word    $B9B9    '64    Del
                        word    $B5B5    '65    End
                        word    $0808    '66    Backspace
                        word    $BABA    '67    Insert
                        word    $0000    '68
                        word    $D1D1    '69    (1)
                        word    $B1B1    '6A    Right 
                        word    $D4D4    '6B    (4)
                        word    $D7D7    '6C    (7)
                        word    $B7B7    '6D    Page Down
                        word    $B4B4    '6E    Home
                        word    $B6B6    '6F    Page Up
                        word    $D0D0    '70    (0)
                        word    $DADA    '71    (.)
                        word    $D2D2    '72    (2)
                        word    $D5D5    '73    (5)
                        word    $D6D6    '74    (6)
                        word    $D8D8    '75    (8)
                        word    $0000    '76    Numlock
                        word    $DFDF    '77    (/)
                        word    $0000    '78
                        word    $DBDB    '79    (Enter)
                        word    $D3D3    '7A    (3)
                        word    $0000    '7B
                        word    $DCDC    '7C    (+)
                        word    $D9D9    '7D    (9)
                        word    $DEDE    '7E    (*)
                        word    $0000    '7F
                        word    $0000    '80
                        word    $0000    '81
                        word    $0000    '82
                        word    $0000    '83
                        word    $DDDD    '84    (-)



keypad1                 byte    $BA, $B5, $B3, $B7, $B0, 0, $B1, $B4, $B2, $B6, $B9, $0D, "+-*/"

keypad2                 byte    "0123456789.", $0D, "+-*/"

'
'
' Uninitialized data
'
dmask                   res     1
cmask                   res     1
stat                    res     1
data                    res     1
x                       res     1
y                       res     1
t                       res     1

sca                     res     1
shiftlock               res     1     

_head                   res     1       'write-only
_present                res     1       'write-only

_dpin                   res     1       'read-only at start
_cpin                   res     1       'read-only at start
_numlock                res     1       'read-only at start
_auto                   res     1       'read-only at start

''
''
''      _________
''      Key Codes
''
''
''      08      Backspace
''      09      Tab
''      0D      Enter
''      1B      Esc
''      20      Space
''      21      !
''      22      "
''      23      #
''      24      $
''      25      %
''      26      &
''      27      '
''      28      (
''      29      )
''      2A      *
''      2B      +
''      2C      ,
''      2D      -
''      2E      .
''      2F      /
''      30..39  0..9
''      3A      :
''      3B      ;
''      3C      <
''      3D      =
''      3E      >
''      3F      ?
''      40      @       
''      41..5A  A..Z
''      5B      [
''      5C      \
''      5D      ]
''      5E      ^
''      5F      _
''      60      `
''      61..7A  a..z
''      7B      {
''      7C      |
''      7D      }
''      7E      ~
''
''      80-AF   unused
''
''      B0      Left Arrow
''      B1      Right Arrow
''      B2      Up Arrow
''      B3      Down Arrow
''      B4      Home
''      B5      End
''      B6      Page Up
''      B7      Page Down
''      B8      unused
''      B9      Delete
''      BA      Insert

''      BB-BF   unused
''
''      C0..CB  F1..F12
''      CC      Print Screen

''      CD-FF   unused
''
'' Note: Driver will buffer up to 15 keystrokes, then ignore overflow.

{{

+------------------------------------------------------------------------------------------------------------------------------+
                                                    TERMS OF USE: MIT License                                                                                                              
+------------------------------------------------------------------------------------------------------------------------------
 Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation     
 files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,            
 modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software
 is furnished to do so, subject to the following conditions:                                                                   
                                                                                                                               
 The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
                                                                                                                               
 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE          
 WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR         
 COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,   
 ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                         
+------------------------------------------------------------------------------------------------------------------------------+
}}