''*******************************
''* PLAY.spin v1.2       6/2016 *
''* Author: Werner L. Schneider *
''*******************************
''
''
VAR

    long cog                     ' cog flag/id

    'DO NOT REARRANGE LONGS
    long ay1_regs                ' 9 contiguous longs
 
    long buff1A_ptr
    long buff1B_ptr
    long buffC_ptr
    long music_ptr

    long flag1A
    long flag1B

    long flagC
    long flagM


OBJ

    ay1 : "AYcog"


PUB start(right, left, musicptr)
    stop

    flag1A := 0
    flag1B := 0
    flagC := 0
    flagM := 0

    ay1_Regs := ay1.start(right, left)    ' Start the emulated AY chip in one cog
 
    ay1.resetRegisters                    ' Reset all AY registers

    ay1.enableTone(true, true, false)   
    ay1.enableNoise(false, false, true) 

    music_ptr := musicptr

    cog := cognew(@entry, @ay1_regs) + 1


PUB stop
    if cog
        cogstop(cog~ - 1)
    longfill(@ay1_regs, 0, 9)


PUB ch1A(offs)
 
    buff1A_ptr := offs
    flag1A := 1


PUB ch1A_stop

    ay1.setVolume(0, 0)
    flag1A := 0


PUB ch1Aflag

    return flag1A


PUB ch1B(offs)
 
    buff1B_ptr := offs
    flag1B := 1


PUB ch1B_stop

    ay1.setVolume(1, 0)
    flag1B := 0


PUB ch1Bflag

    return flag1B


PUB chC(offs)
 
    buffC_ptr := offs
    flagC := 1


PUB chC_stop

    ay1.setVolume(2, 0)
    flagC := 0


PUB chCflag

    return flagC


PUB music

    flagM := 1


PUB music_stop

    flagM := 0

    ay1.resetRegisters                    ' Reset all AY registers

    ay1.enableTone(true, true, false)   
    ay1.enableNoise(false, false, true) 


DAT

'***********************************
'* Assembly language "play" driver *
'***********************************

                        org     
'
' Entry
'

entry                   movd      :par, #_ay1ptr
                        mov       t1, par
                        mov       t2, #9
:par                    mov       0, t1
                        add       :par, dlsb
                        add       t1, #4
                        djnz      t2, #:par

                        mov       _pd, pdelay
 
'*********************************************************************************************************

                        rdlong    _ay1, _ay1ptr           ' AY #1 Reg 0 Fine Tune Ch A
                        mov       _ay12, _ay1
                        add       _ay12, #2               ' AY #1 Reg 2 Fine Tune Ch B
                        mov       _ay16, _ay12
                        add       _ay16, #2 + 2           ' AY #1 Reg 6 Noise Period
                        mov       _ay1e, _ay16
                        add       _ay1e, #6               ' AY #1 Reg 12 Envelope Ccoarse Tune
                        mov       _ay1s, _ay1e
                        add       _ay1s, #1               ' AY #1 Reg 13 Shape Control

                        mov       _ay1volA, _ay1
                        add       _ay1volA, #8            ' AY #1 Reg 8 Volume Ch A
                        mov       _ay1volB, _ay1volA
                        add       _ay1volB, #1            ' AY #1 Reg 9 Volume Ch B
                        mov       _ay1volC, _ay1volB
                        add       _ay1volC, #1            ' AY #1 Reg 10 Volume Ch C

'*********************************************************************************************************

bigloop                 mov       _delay, cnt          
                        add       _delay, _pd

'*********************************************************************************************************

entry1                  rdlong    t1, _flagptr1A          ' Read Ch 1A Flag
                        cmp       t1, #0            wz
             if_z       jmp       #entry2     
                        cmp       t1, #1            wz 
             if_z       add       t1, #1
             if_z       wrlong    t1, _flagptr1A
             if_z       rdlong    _buff1A, _buffptr1A
                        rdlong    _tone, _buff1A
                        add       _buff1A, #4
                        mov       _vol, _tone
                        shr       _vol, #16
                        and       _vol, #$F
                        cmp       _tone, endmask    wz
             if_z       jmp       #ready1A
                        and       _tone, tonemask

                        wrword    _tone, _ay1             ' AY #1 Reg 0/1 Ch A Fine Tune/Coarse Tune
                        wrbyte    _vol, _ay1volA          ' AY #1 Reg 8 Volume Ch A

'*********************************************************************************************************

entry2                  rdlong    t2, _flagptr1B          ' Read Ch 1B Flag
                        cmp       t2, #0            wz
             if_z       jmp       #entry3     
                        cmp       t2, #1            wz 
             if_z       add       t2, #1
             if_z       wrlong    t2, _flagptr1B
             if_z       rdlong    _buff1B, _buffptr1B
                        rdlong    _tone, _buff1B  
                        add       _buff1B, #4
                        mov       _vol, _tone
                        shr       _vol, #16
                        and       _vol, #$F
                        cmp       _tone, endmask    wz
             if_z       jmp       #ready1B
                        and       _tone, tonemask

                        wrword    _tone, _ay12            ' AY #1 Reg 2/3 Ch B Fine Tune/Coarse Tune
                        wrbyte    _vol, _ay1volB          ' AY #1 Reg 9 Volume Ch B

'*********************************************************************************************************

entry3                  rdlong    t3, _flagptrC           ' Read Ch C Flag
                        cmp       t3, #0            wz
             if_z       jmp       #entry4     
                        cmp       t3, #1            wz 
             if_z       add       t3, #1
             if_z       wrlong    t3, _flagptrC
             if_z       rdlong    _buffC, _buffptrC
                        rdlong    _tone, _buffC              ' Flag active
                        add       _buffC, #4
                        mov       _env, _tone
                        shr       _env, #8
                        and       _env, #$FF
                        mov       _shape, _tone
                        shr       _shape, #16
                        and       _shape, #$0F
                        mov       _vol, _tone
                        shr       _vol, #24
                        and       _vol, #$1F
                        cmp       _tone, endmask    wz
             if_z       jmp       #readyC
                        and       _tone, noisemask

                        wrbyte    _tone, _ay16            ' AY #1 Reg 6 Noise Period
                        wrbyte    _env, _ay1e             ' AY #1 Reg 12 Envelope Coarse Tune
                        wrbyte    _shape, _ay1s           ' AY #1 Reg 13 Shape Control
                        wrbyte    _vol, _ay1volC          ' AY #1 Reg 10 Volume Ch C

'*********************************************************************************************************

entry4                  rdlong    t4, _flagptrM           ' Read Music Flag
                        cmp       t4, #1            wz
             if_nz      jmp       #entry5     

musicentry              rdlong    _music, _musicptr

musicloop               mov       _delay, cnt          
                        add       _delay, _pd

                        rdlong    t4, _flagptrM           ' Read Music Flag
                        cmp       t4, #1            wz
             if_nz      jmp       #musicstop     

                        rdlong    _m1, _music
                        cmp       _m1, endmask      wz
             if_z       jmp       #musicstop              ' jmp #musicentry is repeat
                        add       _music, #4
                        rdlong    _m2, _music
                        add       _music, #4
                        rdlong    _m3, _music
                        add       _music, #4

                        mov       _ay, _ay1
                        wrlong    _m1, _ay
                        add       _ay, #4
                        wrlong    _m2, _ay
                        add       _ay, #4
                        wrlong    _m3, _ay

                        waitcnt   _delay, _pd
                        jmp       #musicloop

musicstop               jmp       #bigloop

'*********************************************************************************************************

entry5                  cmp       t1, #0            wz
             if_z       cmp       t2, #0            wz
             if_z       cmp       t3, #0            wz
             if_nz      waitcnt   _delay, _pd        
                        jmp       #bigloop

'*********************************************************************************************************

ready1A                 mov       t1, #0
                        wrlong    t1, _flagptr1A
                        wrbyte    t1, _ay1volA             ' AY #1 Reg 8 Volume Ch A
                        jmp       #entry2

'*********************************************************************************************************

ready1B                 mov       t2, #0
                        wrlong    t2, _flagptr1B
                        wrbyte    t2, _ay1volB             ' AY #1 Reg 9 Volume Ch B
                        jmp       #entry3

'*********************************************************************************************************

readyC                  mov       t3, #0
                        wrlong    t3, _flagptrC
'                        wrbyte    t3, _ayvol3             ' AY-Reg 10 Volume Ch C
                        jmp       #entry4

'*********************************************************************************************************



                        ' Data

dlsb                    long      1 << 9

pdelay                  long      80_000_000 / 50

endmask                 long      $FFFFFFFF
tonemask                long      $00000FFF
noisemask               long      $0000001F

'
' Uninitialized data
'
t1                      res       1
t2                      res       1
t3                      res       1
t4                      res       1

_delay                  res       1
_pd                     res       1
_ay                     res       1

_ay1ptr                 res       1               ' 9 params
_buffptr1A              res       1
_buffptr1B              res       1
_buffptrC               res       1
_musicptr               res       1
_flagptr1A              res       1
_flagptr1B              res       1
_flagptrC               res       1
_flagptrM               res       1

_ay1                    res       1
_ay12                   res       1
_ay16                   res       1
_ay1e                   res       1
_ay1s                   res       1
_ay1volA                res       1
_ay1volB                res       1
_ay1volC                res       1

_buff1A                 res       1
_buff1B                 res       1
_buffC                  res       1

_tone                   res       1
_vol                    res       1
_env                    res       1
_shape                  res       1

_music                  res       1
_m1                     res       1
_m2                     res       1
_m3                     res       1


                        fit     $1f0

{{

Permission is hereby granted, free of charge, to any person obtaining a copy of this
software and associated documentation files (the "Software"), to deal in the Software
without restriction, including without limitation the rights to use, copy, modify,
merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to the following
conditions:

The above copyright notice and this permission notice shall be included in all copies
or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

}}