''**************************************
''* PROPTRIS_1.3_NO_EEPROM.spin 1/2020 *
''* Author: Werner L. Schneider        *
''**************************************
''
'' Inspired by
''
'' http://vadim.oversigma.com/Tetris.htm
'' http://colinfahey.com/tetris/tetris.html
''
''
'' Changes in v1.3
''
'' NO EEPROM Version added. (No EEPROM-Access for Users of PropDOS, Spinix, Sphinx, KyeDOS, etc) 
''
''
'' Changes in v1.2
''
'' F12 for Reboot added.
''
''
'' Changes in v1.1
''
'' VGA Resolutin now 640x480
'' AY-3 8910 Cog added V0.8 (C) 2012 Johannes Ahlebrand
''
''
'' Key's to play                   Action
''
'' Esc                             Exit
''
'' F1                              Pause
''
'' Arrow left                      Move left 
''
'' Arrow right                     Move right 
''
'' Arrow up                        Rotate counterclockwise 
''
'' Arrow down or Space             Drop 
''
'' 1                               Show/Hide next 
''
'' 2                               Speed up 
''
''
'' Have Fun!!!
''
''
CON
    _clkmode   = xtal1 + pll16x
    _xinfreq   = 5_000_000

    video      = 16                     ' P16-P23 VGA

    kbd        = 26                     ' P26 Keyboard Data
    kbc        = 27                     ' P27 Keyboard Clock

  
OBJ

    font:     "FONT"                    ' FONT Driver                              need 1 Cog

'    kb:       "KEYBOARD_DE"             ' Keyboard Driver German                   need 1 Cog
    kb:       "KEYBOARD_EN"             ' Keyboard Driver English                  need 1 Cog

    play:     "PLAY"                    ' AY 3-8910                                need 2 Cog's
    
    show:     "SHOW"                    ' SHOW Driver                              need 1 Cog

    vga:      "VGACOLOR_640x480"        ' VGA Driver                               need 2 Cog's


VAR
 
    byte buffPile[200]
    byte buffNew[200]
    byte buffShow[200]

    byte act_piece[8]
    byte tst_piece[8]

    byte vgabuff[4800]                  ' VGA Text-Buffer 80x30 2400 x 2 Bytes 1. Byte = Character, 2. Byte = Color (bgc * 16 +fgc)
    byte fontbuff[4096]                 ' Font-Buffer 256 Char's 8x16 Pixel

    long color
 
    byte highScore[640]                 ' Table for last 20 Scores
    byte buffScore[32]

    word pieces[8]                      ' Counter-Array for 7 Pieces + 1 x sum

    byte cursor[6]                      ' Cursor info array 
    byte num[9]

    byte name[15]

    long pos                            ' Global Screen-Pointer

    long lines                          ' Full Lines
    long score                          ' Current score

    byte mainlevel

    byte piece                          ' Piece 0-6
    byte piece_x
    byte piece_y

    byte timelev
    byte stage

    byte shownxt
    byte nxt

    byte height

    byte state
    byte level
    byte droptrue
    byte phantom
    byte ph1
    byte ph2

    byte nameptr
    byte path


PUB main     
    Init

    mainlevel := 0

    if printIntro
        mainlevel := 1    
    cls

    repeat
        if mainlevel == 0
            GameLoop 
        else
            DosLoop


PRI GameLoop | key, esc

    if setup
        mainlevel := 1    
        return

    setBigFont

    initGame

    esc := 0
    repeat

        if stage == 0
            if timelev == 0
                droptrue := 0
                piece := nxt
                nxt := initPiece(piece)
                pieces[piece]++
                pieces[7]++
                printStatistic

        key := kb.key
        if key == $1B                                   ' Escape
            esc := 1

        elseif key == $31                               ' "1"
            shownxt ^= 1
            printNext(nxt)

        elseif key == $32                               ' "2"
            if level < 9
                level++
            dec(12, 1, 7, level)                        ' Print actual Level 0-9

        elseif key == $B0                               ' Arrow Left
            testLeft(piece)

        elseif key == $B1                               ' Arrow Right
            testRight(piece)

        elseif key == $C0                               ' "F1" Pause
            show.stop
            cls
            setNormalFont                               ' Switch to 80 x 25
            cursor[2] := %110                           ' Cursor 1 on
            crlf
            sprint(@strPathWork)
            repeat
                key := kb.key
                if key <> 0
                    quit    
            cursor[2] := 0                              ' Cursor 1 off
            cls
            setBigFont                                  ' Switch to 40 x 25
            printMain
            printNext(nxt)
            printStatistic
            show.start(@buffShow, @vgabuff)

        elseif key == $CB                               ' "F12" Reboot
            reboot
            
        if stage == 1
            if key == $B2                               ' Arrow Up
                testRotate(piece)

            elseif key == $B3 or key == $20             ' Arrow Down or Space
                if droptrue == 0
                    drop(piece)

        timelev++
        waitcnt((clkfreq / 1000) * ((10 - level) * 5) + cnt)    
        if timelev == 10
            timelev := 0
            if stage == 0
                stage := 1

            if testDown(piece)

                play.ch1A(@tone5)
                
                if piece_y == 0                             ' Game over

                    show.stop

                    str40(15, 9, 15, 0, @strClear)
                    str40(14, 10, 0, 7, @strOver1)
                    str40(14, 11, 0, 7, @strOver2)
                    str40(14, 12, 0, 7, @strOver1)
                    str40(15, 13, 15, 0, @strClear)
                    str40(0, 26, 2, 0, @strOver3)
                    str40(13, 28, $0F, $00, @strPlay)       ' Print "Play PROPTRIS!"

                    play.ch1A(@tone6)
                    repeat
                        if play.ch1Aflag == 0 
                            play.chC(@noise1)
                            quit

                    repeat
                        key := kb.key
                        if key == $1B                       ' Escape
                            cls
                            mainlevel := 1
                            return

                        elseif key == $52 or key == $72     ' "R" Restart
                            quit

                        elseif key == $CB                   ' "F12" Reboot
                            reboot
                            

                    initGame

                else

                    copyShow2Pile
                    testFullLine
                    stage := 0

            else
                play.ch1A(@tone1)
            
        if esc == 1
            show.stop
            cls
            mainlevel := 1
            quit


PRI DosLoop | i, key, esc

    setNormalFont                                           ' Switch to 80 x 25 

    cursor[2] := %110                                       ' Cursor 1 on

    sprintc($00, $0C, @strSpace2) 
    sprintc($00, $0C, @strPlay2) 
    sprintc($00, $0C, @strSpace2) 
    crlf

    esc := 0
    repeat

        repeat i from 0 to 14
            name[i] := $00 
        nameptr := 0
        sprint(@strPath + (path * 14))

        repeat

            key := kb.key
            if key <> 0
                case key
                    $08:                                    ' Backspace
                        clrBS

                    $1B:                                    ' Escape 
                        if nameptr > 0
                            repeat nameptr
                                clrBS 

                    $0D:                                    ' Enter 
                        if nameptr == 0
                            crlf
                        else
                            name[nameptr] := 0
                            crlf
                            crlf
                            if exec
                                esc := 1
                        quit

                    $20..$7F:                               ' Char($20..$7F)
                        if nameptr < 13

                            cprint(key)
                            name[nameptr] := key
                            nameptr++

                    $CB:                                    ' F12 Reboot 
                        reboot
                        
        if esc == 1
            quit

    cursor[2] := 0                                          ' Cursor 1 off
            
    cls

    mainlevel := 0

    if printIntro
        mainlevel := 1

    cls
    

PRI Init | i, j, addr, magic
''************************************************
''* Start i2c, Keyboard, VGA Cog's, Clear Screen *
''* and set Cursor's                             *
''************************************************

    color := 0                                             ' Palette = 0

    font.start(@fontbmp + 62, @fontbuff)                   ' Start FONT Driver     Offset 62 BMP-Header not used

    play.start(10, 11, @music)                             ' Start right Channel, left Channel, MusicPtr

    kb.start(kbd, kbc)                                     ' Start Keyboard Driver

    flipBitmap                                             ' Flip Bitmap vertical

    setNormalFont                                          ' Switch to 80 x 25 

    vga.start(video, @vgabuff, @cursor, @fontbuff, @color) ' Start Video Driver
    waitcnt(clkfreq * 1 + cnt)                             ' wait 1 second for cogs to start

    cls                                                  ' Clear Screen with $20 Foreground-Color $9 (white), Background-Color $0 (black)

    cursor[0] := 0
    cursor[1] := 0                                          '
    cursor[2] := 0  '%010                                   ' Cursor 1

    cursor[3] := 11
    cursor[4] := 2
    cursor[5] := 0  '%111                                   ' Cursor 2

    shownxt := 1
    phantom := 0
    level := 0
    height := 0

    wordfill(@buffPile, $0000, 100)

    path := 0


DAT
''********************************************************************
''* Functions for Gameplay                                           *
''********************************************************************
''
PRI drop(p) | i, j, k, offs, down

    play.ch1B(@tone3)

    repeat
        j := 0
        k := 0
        offs := (p * 32) + (state * 8)
        repeat i from offs to offs+6 step 2
            tst_piece[j] := (piece_x + tabDown[i]) & $FF
            tst_piece[j+1] := (piece_y + tabDown[i+1]) & $FF
            j += 2
        repeat i from 0 to 6 step 2
            down := tst_piece[i+1] 
            if down == 19
                k := 1
                quit

        if k == 0
            repeat i from 0 to 6 step 2
                down := tst_piece[i]
                if down < 19
                    offs := tst_piece[i] + ((tst_piece[i+1]+1) * 10)    
                    if buffPile[offs] <> 0
                        k := 1
                        quit

        if k == 1
            quit

        piece_y += 1

    score += tabDrop[level]  
    dec(11, 5, $0E, score)                      ' Print actual score
    droptrue := 1

    updatePiece(p)


PRI testFullLine | i, j, l, full

    repeat i from 190 to 0 step 10
        full := 0
        repeat j from 0 to 9
            if buffShow[i+j] > 0 and buffShow[i+j] < 10
                full++
  
        if full < 10
            next
        else
            lines++
            printLines                                  ' Print actual Full Lines

            l := lines // 10 
            if l == 0
                level++

            if level > 9
                level := 9
            dec(12, 1, 7, level)                        ' Print actual Level 0-9

            play.ch1B(@tone4)

            repeat j from 0 to 9
                buffShow[i+j] := 0
                waitcnt((clkfreq / 1000) * 10 + cnt)

            repeat j from i-1 to 0
                buffShow[j+10] := buffShow[j]

            copyShow2Pile

            i := 200


PRI testRotate(p) | i, j, s, offs, tst

    s := state
    s := (s + 1) & $03

    j := 0
    offs := (p * 32) + (s * 8)
    repeat i from offs to offs+6 step 2
       tst_piece[j] := (piece_x + tabPieces[i]) 
       tst_piece[j+1] := (piece_y + tabPieces[i+1]) 
       j += 2

    repeat i from 0 to 6 step 2
        tst := tst_piece[i] 
        if tst < 0 or tst > 9
            return
        tst := tst_piece[i+1]
        if tst > 19
            return

    j := 0
    offs := (p * 32) + (s * 8)
    repeat i from offs to offs+6 step 2
       tst_piece[j] := (piece_x + tabPieces[i]) & $FF  
       tst_piece[j+1] := (piece_y + tabPieces[i+1]) & $FF 
       j += 2

    repeat i from 0 to 6 step 2
        offs := tst_piece[i] + ((tst_piece[i+1]+1) * 10) - 10    
        if buffPile[offs] <> 0
            return 

    play.ch1B(@tone2)

    state := s

    updatePiece(p)


PRI testDown(p) | i, j, offs, down

    j := 0
    offs := (p * 32) + (state * 8)
    repeat i from offs to offs+6 step 2
        tst_piece[j] := (piece_x + tabDown[i]) & $FF
        tst_piece[j+1] := (piece_y + tabDown[i+1]) & $FF
        j += 2

    repeat i from 0 to 6 step 2
        down := tst_piece[i+1] 
        if down == 19
            if droptrue == 0
                score += tabFree[level]
                dec(11, 5, $0E, score)                      ' Print actual score
            return -1

    repeat i from 0 to 6 step 2
        down := tst_piece[i]
        if down < 19
            offs := tst_piece[i] + ((tst_piece[i+1]+1) * 10)    
            if buffPile[offs] <> 0
                if droptrue == 0
                    score += tabFree[level]
                    dec(11, 5, $0E, score)                  ' Print actual score
                return -1

    piece_y += 1

    updatePiece(p)

    return 0

    
PRI testLeft(p) | i, j, offs, left

    j := 0
    offs := (p * 32) + (state * 8)
    repeat i from offs to offs+6 step 2
       tst_piece[j] := (piece_x + tabLeft[i]) & $FF
       tst_piece[j+1] := (piece_y + tabLeft[i+1]) & $FF
       j += 2

    repeat i from 0 to 6 step 2
        left := tst_piece[i] 
        if left == 0
            play.ch1B(@tone0)
            return
        elseif left == 1
            play.ch1B(@tone0)

    repeat i from 0 to 6 step 2
        left := tst_piece[i]
        if left > 0
            offs := left-1 + (tst_piece[i+1] * 10)    
            if buffPile[offs] <> 0
                play.ch1B(@tone0)
                return

    piece_x -= 1

    updatePiece(p)

    
PRI testRight(p) | i, j, offs, right

    j := 0
    offs := (p * 32) + (state * 8)
    repeat i from offs to offs+6 step 2
       tst_piece[j] := (piece_x + tabRight[i]) & $FF
       tst_piece[j+1] := (piece_y + tabRight[i+1]) & $FF
       j += 2

    repeat i from 0 to 6 step 2
        right := tst_piece[i] 
        if right == 9
            play.ch1B(@tone0)
            return
        elseif right == 8
            play.ch1B(@tone0)
            
    repeat i from 0 to 6 step 2
        right := tst_piece[i]
        if right < 9
            offs := right+1 + (tst_piece[i+1] * 10)    
            if buffPile[offs] <> 0
                play.ch1B(@tone0)
                return

    piece_x += 1

    updatePiece(p)


PRI initPiece(p) | i, j, offs, nx

    piece_x := 5
    piece_y := 0
    state := 0
    ph1 := 8
    ph2 := 9
    j := 0
    offs := (p * 32) + (state * 8)

    repeat i from offs to offs+6 step 2
       act_piece[j] := (piece_x + tabPieces[i]) & $FF
       act_piece[j+1] := (piece_y + tabPieces[i+1]) & $FF
       j += 2

    if phantom == 1 and p == 5

        repeat i from 0 to 2 step 2
            offs := act_piece[i] + (act_piece[i+1] * 10) 
            buffShow[offs] := ph2
        repeat i from 4 to 6 step 2
            offs := act_piece[i] + (act_piece[i+1] * 10) 
            buffShow[offs] := ph1

    else
        repeat i from 0 to 6 step 2
            offs := act_piece[i] + (act_piece[i+1] * 10) 
            buffShow[offs] := p+1

    nx := ((||(cnt?)) // 7)             ' 0 - 6

    printNext(nx)

    return nx

PRI printNext(nx)    

    str40(21, 24, 15, 0, @strClear)
    str40(21, 25, 15, 0, @strClear)

    if shownxt == 1
        printPiece(23, 24, nx)


PRI updatePiece(p) | i, j, offs

    wordfill(@buffNew, $0A00, 100)

    copyPile2New

    j := 0
    offs := (p * 32) + (state * 8)
    repeat i from offs to offs+6 step 2
        act_piece[j] := (piece_x + tabPieces[i]) & $FF
        act_piece[j+1] := (piece_y + tabPieces[i+1]) & $FF
        j += 2

    if piece_x // 2 == 0
        ph1 := 9
        ph2 := 8
    else
        ph1 := 8
        ph2 := 9

    if phantom == 1 and p == 5

        repeat i from 0 to 2 step 2
            offs := act_piece[i] + (act_piece[i+1] * 10) 
            buffNew[offs] := ph2
        repeat i from 4 to 6 step 2
            offs := act_piece[i] + (act_piece[i+1] * 10) 
            buffNew[offs] := ph1

    else

        repeat i from 0 to 6 step 2
            offs := act_piece[i] + (act_piece[i+1] * 10) 
            buffNew[offs] := p+1

    longmove(@buffShow, @buffNew, 50)


PRI initGame | i, start

    wordfill(@buffPile, $0000, 100)

    repeat i from 0 to 7
        pieces[i] := 0

    stage := 0
    timelev := 0
    state := 0
    lines := 0
    score := 0

    printMain

    printStatistic

    wordfill(@buffNew, $0A00, 100)

    if height > 0
        if height == 1
            start := 160                        ' Print Block  4 x 10
        elseif height == 2
            start := 100                        ' Print Block 10 x 10
        elseif height == 3
            start := 60                         ' Print Block 14 x 10
            
        repeat i from start to 199
            byte[@buffnew + i] := ((||(cnt?)) // 7) + 1
        repeat i from start to 190 step 10
            byte[@buffnew + i + ((||(cnt?)) // 10)] := 0
        longmove(@buffShow, @buffNew, 50)
        copyShow2Pile

    longmove(@buffShow, @buffNew, 50)

    show.start(@buffShow, @vgabuff)
    
    nxt := ((||(cnt?)) // 7) 


PRI copyPile2New | i

    repeat i from 0 to 199
        if buffPile[i] <> 0
            buffNew[i] := buffPile[i]


PRI copyShow2Pile | i

    repeat i from 0 to 199
        if buffShow[i] > 0 and buffShow[i] < 10 
            buffPile[i] := buffShow[i]
        else
            buffPile[i] := 0


DAT
''********************************************************************
''* Functions for Pseudo-DOS ;-)                                     *
''********************************************************************
''
PRI exec | i

    repeat i from 0 to 14
        name[i] := ucase(name[i])
        
    if strcomp(@strCmd1, @name)                             ' "CLS"
        cls

    if strcomp(@strCmd2, @name)                             ' "PROPTRIS"
        return -1

    if strcomp(@strCmd3, @name)                             ' "EXIT"
        return -1

    if strcomp(@strCmd4, @name)                             ' "CD.."
        path := 1

    if strcomp(@strCmd5, @name)                             ' "CD .."
        path := 1

    if strcomp(@strCmd6, @name)                             ' "CD\"
        path := 1

    if strcomp(@strCmd7, @name)                             ' "CD \"
        path := 1

    if strcomp(@strCmd8, @name)                             ' "CD PROPTRIS"
        path := 0

    if strcomp(@strCmd9, @name)                             ' "CD \PROPTRIS"
        path := 0                                           

    if strcomp(@strCmd10, @name)                            ' "DIR"
        sprint(@strVolume)
        sprint(@strDir + (path * 138))

    return 0


PRI clrBS

    if nameptr > 0
        if cursor[0] == 0
            cursor[1]--
            cursor[0] := 79
        else
            cursor[0]--
        pos := (cursor[0] + cursor[1] * 80) * 2
        print80($07, $00, $20)
        nameptr--


PRI crlf

    cprint($0D)
    cprint($0A)


PRI ucase(c)

    if c < 97
        return c
    elseif c > 122
        return c
    else
        return c-32    


PRI cprint(c)

    case c
        $08:
            if cursor[0] == 0
                cursor[1]--
                cursor[0] := 79
            else
                cursor[0]--
            pos := (cursor[0] + cursor[1] * 80) * 2
            print80($07, $00, $20)
        $0D:                                        ' CR
            cursor[0] := 0
        $0A:                                        ' LF
            if cursor[1] == 24
                scroll
            else
                cursor[1]++
        other: 
            pos := (cursor[0]++ + cursor[1] * 80) * 2
            print80($07, $00, c)
            if cursor[0] == 80
                cursor[0] := 0
                if cursor[1] == 24
                    scroll
                else
                    cursor[1]++


PRI cprintc(fgc, bgc, c)

    case c
        $08:
            if cursor[0] == 0
                cursor[1]--
                cursor[0] := 79
            else
                cursor[0]--
            pos := (cursor[0] + cursor[1] * 80) * 2
            print80($09, $00, $20)
        $0D:                                        ' CR
            cursor[0] := 0
        $0A:                                        ' LF
            if cursor[1] == 24
                scroll
            else
                cursor[1]++
        other: 
            pos := (cursor[0]++ + cursor[1] * 80) * 2
            print80(fgc, bgc, c)
            if cursor[0] == 80
                cursor[0] := 0
                if cursor[1] == 24
                    scroll
                else
                    cursor[1]++


PRI sprint(s) | c, ptr

    repeat 
        c := byte[s++]
        if c == 0
            quit
        cprint(c)


PRI sprintc(fgc, bgc, s) | c, ptr

    repeat 
        c := byte[s++]
        if c == 0
            quit
        cprintc(fgc, bgc, c)


PRI scroll | i, src, dest

    repeat i from 1 to 24    
        src := i * 160    
        dest := (i-1) * 160    
        bytemove(@vgabuff[dest], @vgabuff[src], 160)

    wordfill(@vgabuff[3840], $0920, 80) 


DAT
''********************************************************************
''* Functions for Setup                                              *
''********************************************************************
''
PRI setup | i, key, lev

    str80(30, 0, $08, 0, @strSetup1)
    str80(27, 3, $07, 0, @strSetup2)
    str80(44, 3, $09, 0, @strSetup3)
    str80(25, 7, $07, 0, @strSetup4)
    str80(42, 7, $07, 0, @strSetup5)
    str80(24, 11, $07, 0, @strSetup6)
    str80(42, 11, $07, 0, @strSetup7)
    str80(32, 13, $0F, 0, @strSetup8)
    str80(33, 17, $0F, 0, @strSetup9)
    repeat i from 0 to 9
        printxy80(16 + (i * 5), 15, $06, 0, i + $30)
    str80(28, 19, $06, 0, @strSetup10)

    str80(22, 29, $0F, 0, @strSetup11a)         ' "Press "
    str80(28, 29, $08, 0, @strSetup11b)         ' "SPACE"
    str80(33, 29, $0F, 0, @strSetup11c)         ' " to Start, "
    str80(44, 29, $08, 0, @strSetup11d)         ' "F12"
    str80(47, 29, $0F, 0, @strSetup11e)         ' " to Reboot"

    box1(22, 2, $0C, 0)
    box1(39, 2, $09, 0)

    lev := 0

    clrSetup
    printSetup(lev)

    repeat
        key := kb.key
        if key <> 0

            play.chC(@noise0)

            if key == $1B                       ' Escape
                cls
                return -1

            elseif key == $20                   ' Space
                quit

            elseif key == $B0                   ' Arrow Left
                if lev == 0
                    shownxt ^= 1
                elseif lev == 1
                    phantom ^= 1
                elseif lev == 2
                    if level == 0
                        level := 9
                    else
                        level--
                elseif lev == 3
                    if height == 0
                        height := 3
                    else
                        height--

            elseif key == $B1                   ' Arrow Right
                if lev == 0
                    shownxt ^= 1
                elseif lev == 1
                    phantom ^= 1
                elseif lev == 2
                    if level == 9
                        level := 0
                    else
                        level++
                elseif lev == 3
                    if height == 3
                        height := 0
                    else
                        height++

            elseif key == $B2                   ' Arrow Up
                if lev == 0
                    lev := 3
                else
                    lev--
                clrSetup

            elseif key == $B3                   ' Arrow Down
                if lev == 3
                    lev := 0
                else
                    lev++
                clrSetup

            elseif key == $CB                   ' F12
                reboot

            printSetup(lev)

    cls
    return 0


PRI printSetup(lev) | i

    if lev == 0
        if shownxt == 1
            box1(22, 6, $0C, 1)
            box1(39, 6, $0E, 0)
        else
            box1(22, 6, $0E, 0)
            box1(39, 6, $0C, 1)

    elseif lev == 1
        if phantom == 1
            box1(22, 10, $0C, 1)
            box1(39, 10, $0E, 0)
        else
            box1(22, 10, $0E, 0)
            box1(39, 10, $0C, 1)

    elseif lev == 2
        repeat i from 0 to 9
            if i == level
                box2(14 + (i * 5), 14, $0C, 1)        
            else
                box2(14 + (i * 5), 14, $0E, 0)        

    elseif lev == 3
        repeat i from 0 to 3
            if i == height
                box3(25 + (i * 7), 18, $0C, 1)        
            else
                box3(25 + (i * 7), 18, $0E, 0)        


PRI clrSetup | i

    if shownxt == 1
        box1(22, 6, $0C, 0)
        box1(39, 6, $0E, 0)
    else
        box1(22, 6, $0E, 0)
        box1(39, 6, $0C, 0)

    if phantom == 1
        box1(22, 10, $0C, 0)
        box1(39, 10, $0E, 0)
    else
        box1(22, 10, $0E, 0)
        box1(39, 10, $0C, 0)

    repeat i from 0 to 9
        if i == level
            box2(14 + (i * 5), 14, $0C, 0)        
        else
            box2(14 + (i * 5), 14, $0E, 0)        

    repeat i from 0 to 3
        if i == height
            box3(25 + (i * 7), 18, $0C, 0)        
        else
            box3(25 + (i * 7), 18, $0E, 0)        


DAT
''********************************************************************
''* Functions for Font                                               *
''********************************************************************
''
PRI flipBitmap | sa, sb, buffa[16] 

    sa := 62                                          ' Flip Bitmap vertical
    sb := 2094
    repeat 64
        bytemove(@buffa, @fontbmp + sa, 16)
        bytemove(@fontbmp + sa, @fontbmp + sb, 16)
        bytemove(@fontbmp + sb, @buffa, 16)
        sa += 16
        sb -= 16

        
PRI setNormalFont

    color := 0

    font.normal


PRI setBigFont

    color := 10

    font.big

    
DAT
''********************************************************************
''* Functions for Statistic                                          *
''********************************************************************
''
PRI printStatistic | i

    dec(12, 1, 7, level)                        ' Print actual Level 0-9
    printLines                                  ' Print actual Full Lines
    dec(11, 5, $0E, score)                      ' Print actual score

    repeat i from 0 to 6
        dec(39, 3 + (i * 3), i+1, pieces[i])
    dec(39, 26, 7, pieces[7])


PRI dec(x, y, fgc, val) | i, p, npos 

    repeat i from 0 to 8
        num[i] := 0

    npos := ((x * 2) + y * 80) * 2
    pos := 0
    prn(val)

    p := pos-1
    repeat pos
        vgabuff[npos] := num[p]
        vgabuff[npos+1] := fgc
        vgabuff[npos+2] := num[p] + 128
        vgabuff[npos+3] := fgc
        npos -= 4
        p--


PRI printLines | i, p, npos

    repeat i from 0 to 8
        num[i] := 0

    pos := 0
    prn(lines)

    npos := 528
    if lines > 99
        npos += 4

    p := pos-1
    repeat pos
        vgabuff[npos] := num[p]
        vgabuff[npos+1] := 7
        vgabuff[npos+2] := num[p] + 128
        vgabuff[npos+3] := 7
        npos -= 4
        p--


PRI prn(val) | dig

    dig := 48 + (val // 10)
    val := val/10
    if val > 0
        prn(val)
    num[pos++] := dig


DAT
''********************************************************************
''* Functions for Main-Screen                                        *
''********************************************************************
''
PRI printMain | i
    
    str40(0, 1, $07, $00, @strYour)              ' Print "Your level"
    str40(0, 3, $07, $00, @strFull)              ' Print "Full lines"

    str40(1, 5, $07, $00, @strScore)             ' Print "Score"
    str40(6, 5, $0E, $00, string("      "))      ' Print "      "

    str40(3, 8, $07, $00, @strHelp)              ' Print "H E L P"

    str40(0, 10, $07, $00, @strPause)            ' Print "F1:Pause"
    str40(0, 12, $07, $00, @strReboot)           ' Print "F12:Reboot"
    str40(0, 14, $07, $00, @strLeft)             ' Print "Left"
    str40(0, 16, $07, $00, @strRight)            ' Print "Right"
    str40(0, 18, $07, $00, @strRotate)           ' Print "Rotate"
    str40(0, 20, $07, $00, @strDrop)             ' Print "Drop"
    str40(0, 22, $07, $00, @strDraw)             ' Print "1:Draw next"
    str40(0, 24, $07, $00, @strSpeed)            ' Print "2:Speed up"
    str40(0, 26, $07, $00, @strClr)
    str40(0, 28, $07, $00, @strClr)

    repeat i from 1 to 20
        str40(14, i, $0D, $00, @strField1)       ' Print "*          *"
    str40(14, 21, $0D, $00, @strField2)          ' Print "************"

    str40(15, 24, $07, $00, @strNext)            ' Print "Next:"

    str40(29, 1, $07, $00, @strStat)             ' Print "STATISTICS"

    repeat i from 0 to 6
        str40(35, 3 + (i * 3), i+1, $00, @strSt) ' Print "-   0"

    printPiece(29,  3, 0)
    printPiece(32,  6, 1)
    printPiece(29,  9, 2)
    printPiece(32, 12, 3)
    printPiece(29, 15, 4)

    if phantom == 0
        printPiece(33, 18, 5)
    else
        printxy40(32, 18, 0, 6, $70)             ' Print "p"
        printxy40(33, 18, 6, 0, $06)             ' Print " "
        printxy40(32, 19, 6, 0, $06)             ' Print " "
        printxy40(33, 19, 0, 6, $68)             ' Print "h"

    printPiece(29, 21, 6)


    str40(28, 24, $07, $00, @strSum1)            ' Print "------------"
    str40(28, 26, $07, $00, @strSum2)            ' Print "Sum    :   0"

    str40(13, 28, $02, $00, @strPlay)            ' Print "Play PROPTRIS!"
    

PRI printPiece(x, y, p) | i, offs

    offs := p * 32
    repeat i from offs to offs+6 step 2
       printxy40((x + tabPieces[i]) & $FF, (y + tabPieces[i+1]) & $FF, p+1, 0, p+1)
    

DAT
''********************************************************************
''* Functions for Intro-Screen                                       *
''********************************************************************
''
PRI printIntro | i, j, key, ec

    play.music

    str80(0, 29, $09, 0, @strWS)

    repeat i from 0 to 37 
        repeat j from 0 to 11
            printTetris1(j+1, tabColor[i], @strTetris1+(j*80)) 
            waitcnt((clkfreq / 1000) * 38 + cnt)               
            key := kb.key
            if key == $1B                          ' ESC
                play.music_stop
                return -1
            elseif key == $CB                      ' F12
                reboot
            elseif key <> 0
                play.music_stop
                return 0

        repeat j from 0 to 4
            printTetris2(j+16, tabColor[i], @strTetris2+(j*44)) 
            waitcnt((clkfreq / 1000) * 38 + cnt)               
            key := kb.key
            if key == $1B                          ' ESC
                play.music_stop                    
                return -1
            elseif key == $CB                      ' F12
                reboot
            elseif key <> 0
                play.music_stop
                return 0

        repeat j from 0 to 1
            printTetris3(j+23, tabColor[i], @strTetris3+(j*38)) 
            waitcnt((clkfreq / 1000) * 38 + cnt)               
            key := kb.key
            if key == $1B                          ' ESC
                play.music_stop                    
                return -1
            elseif key == $CB                      ' F12
                reboot
            elseif key <> 0
                play.music_stop
                return 0

    play.music_stop
    return 0


PRI printTetris1(y, bgc, string_ptr) | val

    pos := (y * 80) * 2
    repeat 80
        val := byte[string_ptr++]
        if val == $20
            print80($08, 0, val)
        else
            print80($08, bgc, val)


PRI printTetris2(y, bgc, string_ptr)

    pos := (17 + y * 80) * 2
    repeat 44
        print80($08, bgc, byte[string_ptr++])


PRI printTetris3(y, fgc, string_ptr)

    pos := (20 + y * 80) * 2
    repeat 38
        print80(fgc, 0, byte[string_ptr++])


DAT
''********************************************************************
''* Miscellaneous Functions                                          *
''********************************************************************
''
PRI cls

    wordfill(@vgabuff, $0720, 2400)                        ' Clear Screen with $20 Foreground-Color $7 (white), Background-Color $0 (black)
    cursor[0] := 0 
    cursor[1] := 0


PRI box1(x, y, c, f) | i

    printxy80(x, y, c, 0, $10+(f * 6))
    repeat i from 0 to 14
        printxy80(x+1+i, y, c, 0, $14+(f * 6))
    printxy80(x+16, y, c, 0, $11+(f * 6))

    printxy80(x, y+1, c, 0, $15+(f * 6))
    printxy80(x+16, y+1, c, 0, $15+(f * 6))

    printxy80(x, y+2, c, 0, $12+(f * 6))
    repeat i from 0 to 14
        printxy80(x+1+i, y+2, c, 0, $14+(f * 6))
    printxy80(x+16, y+2, c, 0, $13+(f * 6))


PRI box2(x, y, c, f) | i

    printxy80(x, y, c, 0, $10+(f * 6))
    repeat i from 0 to 2
        printxy80(x+1+i, y, c, 0, $14+(f * 6))
    printxy80(x+4, y, c, 0, $11+(f * 6))

    printxy80(x, y+1, c, 0, $15+(f * 6))
    printxy80(x+4, y+1, c, 0, $15+(f * 6))

    printxy80(x, y+2, c, 0, $12+(f * 6))
    repeat i from 0 to 2
        printxy80(x+1+i, y+2, c, 0, $14+(f * 6))
    printxy80(x+4, y+2, c, 0, $13+(f * 6))


PRI box3(x, y, c, f) | i

    printxy80(x, y, c, 0, $10+(f * 6))
    repeat i from 0 to 4
        printxy80(x+1+i, y, c, 0, $14+(f * 6))
    printxy80(x+6, y, c, 0, $11+(f * 6))

    printxy80(x, y+1, c, 0, $15+(f * 6))
    printxy80(x+6, y+1, c, 0, $15+(f * 6))

    printxy80(x, y+2, c, 0, $12+(f * 6))
    repeat i from 0 to 4
        printxy80(x+1+i, y+2, c, 0, $14+(f * 6))
    printxy80(x+6, y+2, c, 0, $13+(f * 6))


PRI print40(fgc, bgc, char) 
''************************************************
''* Print Character char at current pos          *
''* with Foreground-Color fgc and                *
''*      Background-Color bgc                    *
''************************************************

    vgabuff[pos++] := char
    vgabuff[pos++] := bgc * 16 + fgc
    vgabuff[pos++] := char + 128
    vgabuff[pos++] := bgc * 16 + fgc


PRI printxy40(x, y, fgc, bgc, char) 
''************************************************
''* Print Character char at x, y                 *
''* with Foreground-Color fgc and                *
''*      Background-Color bgc                    *
''************************************************

    pos := ((x *2 ) + y * 80) * 2
    print40(fgc, bgc, char)


PRI print80(fgc, bgc, char) 
''************************************************
''* Print Character char at current pos          *
''* with Foreground-Color fgc and                *
''*      Background-Color bgc                    *
''************************************************

    vgabuff[pos++] := char
    vgabuff[pos++] := bgc * 16 + fgc


PRI printxy80(x, y, fgc, bgc, char) 
''************************************************
''* Print Character char at x, y                 *
''* with Foreground-Color fgc and                *
''*      Background-Color bgc                    *
''************************************************

    pos := (x + y * 80) * 2
    print80(fgc, bgc, char)


PRI str40(x, y, fgc, bgc, string_ptr)
''************************************************
''* Print String at x, y                         *
''* with Foreground-Color fgc and                *
''*      Background-Color bgc                    *
''************************************************

    pos := ((x * 2) + y * 80) * 2
    repeat strsize(string_ptr)
        print40(fgc, bgc, byte[string_ptr++])


PRI str80(x, y, fgc, bgc, string_ptr)
''************************************************
''* Print String at x, y                         *
''* with Foreground-Color fgc and                *
''*      Background-Color bgc                    *
''************************************************

    pos := (x + y * 80) * 2
    repeat strsize(string_ptr)
        print80(fgc, bgc, byte[string_ptr++])


DAT


strTetris1        byte $1C, $1F, $1F, $1F, $1F, $1F, $1F, $1D, $20, $20, $1C, $1F, $1F, $1F, $1F, $1F, $1F, $1F, $1D, $20 
                  byte $20, $20, $1C, $1F, $1F, $1F, $1F, $1F, $1D, $20, $20, $1C, $1F, $1F, $1F, $1F, $1F, $1F, $1D, $20   
                  byte $20, $1C, $1F, $1F, $1F, $1F, $1F, $1F, $1F, $1D, $20, $1C, $1F, $1F, $1F, $1F, $1F, $1F, $1F, $1D
                  byte $20, $20, $1C, $1F, $1F, $1F, $1F, $1F, $1D, $20, $20, $1C, $1F, $1F, $1F, $1F, $1F, $1D, $20, $20  

                  byte $1C, $00, $00, $1E, $1E, $1E, $00, $00, $1D, $20, $1C, $00, $00, $1E, $1E, $1E, $1E, $00, $00, $1D
                  byte $20, $1C, $00, $00, $1E, $1E, $1E, $00, $00, $1D, $20, $1C, $00, $00, $1E, $1E, $1E, $00, $00, $1D
                  byte $20, $1C, $1E, $1E, $00, $00, $00, $1E, $1E, $1D, $20, $1C, $00, $00, $1E, $1E, $1E, $1E, $00, $00
                  byte $1D, $20, $20, $20, $1C, $00, $1D, $20, $20, $20, $1C, $00, $00, $1E, $1E, $1E, $00, $00, $1D, $20

                  byte $1C, $00, $1D, $20, $20, $20, $1C, $00, $1D, $20, $1C, $00, $1D, $20, $20, $20, $20, $1C, $00, $1D
                  byte $20, $1C, $00, $1D, $20, $20, $20, $1C, $00, $1D, $20, $1C, $00, $1D, $20, $20, $20, $1C, $00, $1D
                  byte $20, $20, $20, $20, $1C, $00, $1D, $20, $20, $20, $20, $1C, $00, $1D, $20, $20, $20, $20, $1C, $00                   
                  byte $1D, $20, $20, $20, $1C, $00, $1D, $20, $20, $20, $1C, $00, $1D, $20, $20, $20, $1C, $00, $1D, $20

                  byte $1C, $00, $1D, $20, $20, $20, $1C, $00, $1D, $20, $1C, $00, $1D, $20, $20, $20, $20, $1C, $00, $1D
                  byte $20, $1C, $00, $1D, $20, $20, $20, $1C, $00, $1D, $20, $1C, $00, $1D, $20, $20, $20, $1C, $00, $1D
                  byte $20, $20, $20, $20, $1C, $00, $1D, $20, $20, $20, $20, $1C, $00, $1D, $20, $20, $20, $20, $1C, $00                   
                  byte $1D, $20, $20, $20, $1C, $00, $1D, $20, $20, $20, $1C, $00, $1D, $20, $20, $20, $1E, $1E, $1E, $20
                   
                  byte $1C, $00, $1D, $20, $20, $20, $1C, $00, $1D, $20, $1C, $00, $1D, $20, $20, $20, $20, $1C, $00, $1D
                  byte $20, $1C, $00, $1D, $20, $20, $20, $1C, $00, $1D, $20, $1C, $00, $1D, $20, $20, $20, $1C, $00, $1D
                  byte $20, $20, $20, $20, $1C, $00, $1D, $20, $20, $20, $20, $1C, $00, $1D, $20, $20, $20, $20, $1C, $00                   
                  byte $1D, $20, $20, $20, $1C, $00, $1D, $20, $20, $20, $1C, $00, $1D, $20, $20, $20, $20, $20, $20, $20

                  byte $1C, $00, $00, $1F, $1F, $1F, $00, $00, $1D, $20, $1C, $00, $00, $1F, $1F, $1F, $1F, $00, $1D, $20 
                  byte $20, $1C, $00, $1D, $20, $20, $20, $1C, $00, $1D, $20, $1C, $00, $00, $1F, $1F, $1F, $00, $00, $1D
                  byte $20, $20, $20, $20, $1C, $00, $1D, $20, $20, $20, $20, $1C, $00, $00, $1F, $1F, $1F, $1F, $00, $1D
                  byte $20, $20, $20, $20, $1C, $00, $1D, $20, $20, $20, $20, $1C, $00, $1F, $1F, $1F, $1D, $20, $20, $20 

                  byte $1C, $00, $00, $1E, $1E, $1E, $1E, $1D, $20, $20, $1C, $00, $00, $1E, $1E, $00, $1D, $20, $20, $20 
                  byte $20, $1C, $00, $1D, $20, $20, $20, $1C, $00, $1D, $20, $1C, $00, $00, $1E, $1E, $1E, $1E, $1D, $20
                  byte $20, $20, $20, $20, $1C, $00, $1D, $20, $20, $20, $20, $1C, $00, $00, $1E, $1E, $00, $1D, $20, $20 
                  byte $20, $20, $20, $20, $1C, $00, $1D, $20, $20, $20, $20, $20, $1C, $1E, $1E, $1E, $00, $1D, $20, $20

                  byte $1C, $00, $1D, $20, $20, $20, $20, $20, $20, $20, $1C, $00, $1D, $20, $20, $1C, $00, $1D, $20, $20
                  byte $20, $1C, $00, $1D, $20, $20, $20, $1C, $00, $1D, $20, $1C, $00, $1D, $20, $20, $20, $20, $20, $20
                  byte $20, $20, $20, $20, $1C, $00, $1D, $20, $20, $20, $20, $1C, $00, $1D, $20, $20, $1C, $00, $1D, $20                   
                  byte $20, $20, $20, $20, $1C, $00, $1D, $20, $20, $20, $20, $20, $20, $20, $20, $20, $1C, $00, $1D, $20

                  byte $1C, $00, $1D, $20, $20, $20, $20, $20, $20, $20, $1C, $00, $1D, $20, $20, $1C, $00, $1D, $20, $20
                  byte $20, $1C, $00, $1D, $20, $20, $20, $1C, $00, $1D, $20, $1C, $00, $1D, $20, $20, $20, $20, $20, $20
                  byte $20, $20, $20, $20, $1C, $00, $1D, $20, $20, $20, $20, $1C, $00, $1D, $20, $20, $1C, $00, $1D, $20                   
                  byte $20, $20, $20, $20, $1C, $00, $1D, $20, $20, $20, $1F, $1F, $1F, $20, $20, $20, $1C, $00, $1D, $20
                  
                  byte $1C, $00, $1D, $20, $20, $20, $20, $20, $20, $20, $1C, $00, $1D, $20, $20, $20, $1C, $00, $1D, $20
                  byte $20, $1C, $00, $1D, $20, $20, $20, $1C, $00, $1D, $20, $1C, $00, $1D, $20, $20, $20, $20, $20, $20
                  byte $20, $20, $20, $20, $1C, $00, $1D, $20, $20, $20, $20, $1C, $00, $1D, $20, $20, $20, $1C, $00, $1D                   
                  byte $20, $20, $20, $20, $1C, $00, $1D, $20, $20, $20, $1C, $00, $1D, $20, $20, $20, $1C, $00, $1D, $20

                  byte $1C, $00, $1D, $20, $20, $20, $20, $20, $20, $20, $1C, $00, $1D, $20, $20, $20, $1C, $00, $1D, $20
                  byte $20, $1C, $00, $00, $1F, $1F, $1F, $00, $00, $1D, $20, $1C, $00, $1D, $20, $20, $20, $20, $20, $20
                  byte $20, $20, $20, $20, $1C, $00, $1D, $20, $20, $20, $20, $1C, $00, $1D, $20, $20, $20, $1C, $00, $1D                   
                  byte $20, $20, $20, $20, $1C, $00, $1D, $20, $20, $20, $1C, $00, $00, $1F, $1F, $1F, $00, $00, $1D, $20

                  byte $1E, $1E, $1E, $20, $20, $20, $20, $20, $20, $20, $1E, $1E, $1E, $20, $20, $20, $1C, $1E, $1E, $1D
                  byte $20, $20, $1C, $1E, $1E, $1E, $1E, $1E, $1D, $20, $20, $1E, $1E, $1E, $20, $20, $20, $20, $20, $20  
                  byte $20, $20, $20, $20, $1E, $1E, $1E, $20, $20, $20, $20, $1E, $1E, $1E, $20, $20, $20, $1C, $1E, $1E
                  byte $1D, $20, $1C, $1E, $1E, $1E, $1E, $1E, $1D, $20, $20, $1C, $1E, $1E, $1E, $1E, $1E, $1D, $20, $20 


strTetris2        byte $10, $14, $14, $14, $14, $14, $14, $14, $14, $14, $14, $14, $14, $14, $14, $14, $14, $14, $14, $14, $14, $14
                  byte $14, $14, $14, $14, $14, $14, $14, $14, $14, $14, $14, $14, $14, $14, $14, $14, $14, $14, $14, $14, $14, $11
                  byte $15, " Game by Werner L. Schneider, inspired by ", $15
                  byte $15, "                  TETRIS                  ", $15
                  byte $15, "    from  A. Pajitnov  &  V. Gerasimov    ", $15
                  byte $12, $14, $14, $14, $14, $14, $14, $14, $14, $14, $14, $14, $14, $14, $14, $14, $14, $14, $14, $14, $14, $14
                  byte $14, $14, $14, $14, $14, $14, $14, $14, $14, $14, $14, $14, $14, $14, $14, $14, $14, $14, $14, $14, $14, $13


strTetris3        byte "(C) AcademySoft CCAS USSR Moscow, 1986"
                  byte "    (C) ELORG COPYRIGHT  1987,1988    "


tabColor          byte $01, $02, $03, $04, $05, $06, $09, $01, $02, $03, $04, $05, $06, $09, $01, $02, $03, $04, $05
                  byte $06, $09, $01, $02, $03, $04, $05, $06, $09, $01, $02, $03, $04, $05, $06, $09, $01, $02, $03


tabFree           byte 6, 9, 12, 15, 18, 21, 24, 27, 30, 33


tabDrop           byte 24, 27, 30, 33, 36, 39, 42, 45, 48, 51


tabPieces         byte  0,  0,  1,  0, -1,  0, -1,  1      ' Piece 0  State 0
                  byte  0,  0,  0, -1,  0,  1,  1,  1      '          State 1                    
                  byte  0,  0, -1,  0,  1,  0,  1, -1      '          State 2   
                  byte  0,  0,  0, -1, -1, -1,  0,  1      '          State 3   

                  byte  0,  0,  1,  0, -1,  0, -2,  0      ' Piece 1  State 0   
                  byte  0,  0,  0, -1,  0,  1,  0,  2      '          State 1   
                  byte  0,  0,  1,  0, -1,  0, -2,  0      '          State 2   
                  byte  0,  0,  0, -1,  0,  1,  0,  2      '          State 3   

                  byte  0,  0,  1,  0, -1,  0,  0,  1      ' Piece 2  State 0    
                  byte  0,  0,  0, -1,  0,  1,  1,  0      '          State 1   
                  byte  0,  0,  1,  0,  0, -1, -1,  0      '          State 2   
                  byte  0,  0,  0,  1,  0, -1, -1,  0      '          State 3   

                  byte  0,  0,  1,  0,  0,  1, -1,  1      ' Piece 3  State 0    
                  byte  0,  0,  0, -1,  1,  0,  1,  1      '          State 1   
                  byte  0,  0,  1,  0,  0,  1, -1,  1      '          State 2   
                  byte  0,  0,  0, -1,  1,  0,  1,  1      '          State 3   

                  byte  0,  0, -1,  0,  0,  1,  1,  1      ' Piece 4  State 0   
                  byte  0,  0,  0,  1,  1,  0,  1, -1      '          State 1   
                  byte  0,  0, -1,  0,  0,  1,  1,  1      '          State 2   
                  byte  0,  0,  0,  1,  1,  0,  1, -1      '          State 3   

                  byte  0,  0,  0,  1, -1,  0, -1,  1      ' Piece 5  State 0   
                  byte  0,  0,  0,  1, -1,  0, -1,  1      '          State 1   
                  byte  0,  0,  0,  1, -1,  0, -1,  1      '          State 2   
                  byte  0,  0,  0,  1, -1,  0, -1,  1      '          State 3   

                  byte  0,  0,  1,  0,  1,  1, -1,  0      ' Piece 6  State 0   
                  byte  0,  0,  0,  1,  0, -1,  1, -1      '          State 1   
                  byte  0,  0,  1,  0, -1,  0, -1, -1      '          State 2   
                  byte  0,  0,  0, -1,  0,  1, -1,  1      '          State 3   


tabLeft           byte -1,  0, -1,  1, -1,  0, -1, -1      ' Piece 0  State 0   
                  byte  0, -1,  0,  0,  0,  1,  0,  1      '          State 1
                  byte  1, -1, -1,  0,  1, -1, -1,  0      '          State 2   
                  byte -1, -1,  0,  0,  0,  1,  0,  1      '          State 3   

                  byte -2,  0, -2,  0, -2,  0, -2,  0      ' Piece 1  State 0   
                  byte  0, -1,  0,  0,  0,  1,  0,  2      '          State 1   
                  byte -2,  0, -2,  0, -2,  0, -2,  0      '          State 2   
                  byte  0, -1,  0,  0,  0,  1,  0,  2      '          State 3   

                  byte -1,  0,  0,  1, -1,  0,  0,  1      ' Piece 2  State 0   
                  byte  0, -1,  0,  0,  0,  1,  0,  1      '          State 1   
                  byte  0, -1, -1,  0,  0, -1, -1,  0      '          State 2   
                  byte  0, -1, -1,  0,  0,  1,  0,  1      '          State 3   

                  byte  0,  0, -1,  1,  0,  0, -1,  1      ' Piece 3  State 0   
                  byte  0, -1,  0,  0,  1,  1,  1,  1      '          State 1   
                  byte  0,  0, -1,  1,  0,  0, -1,  1      '          State 2   
                  byte  0, -1,  0,  0,  1,  1,  1,  1      '          State 3   

                  byte -1,  0,  0,  1, -1,  0,  0,  1      ' Piece 4  State 0   
                  byte  1, -1,  0,  0,  0,  1,  0,  1      '          State 1   
                  byte -1,  0,  0,  1, -1,  0,  0,  1      '          State 2   
                  byte  1, -1,  0,  0,  0,  1,  0,  1      '          State 3   

                  byte -1,  0, -1,  1, -1,  0, -1,  1      ' Piece 5  State 0   
                  byte -1,  0, -1,  1, -1,  0, -1,  1      '          State 1   
                  byte -1,  0, -1,  1, -1,  0, -1,  1      '          State 2   
                  byte -1,  0, -1,  1, -1,  0, -1,  1      '          State 3   

                  byte -1,  0,  1,  1, -1,  0,  1,  1      ' Piece 6  State 0    
                  byte  0, -1,  0,  0,  0,  1,  0,  1      '          State 1   
                  byte -1, -1, -1,  0, -1, -1, -1,  0      '          State 2   
                  byte  0, -1,  0,  0, -1,  1, -1,  1      '          State 3   


tabRight          byte  1,  0, -1,  1,  1,  0, -1,  1      ' Piece 0  State 0     
                  byte  0, -1,  0,  0,  1,  1,  1,  1      '          State 1   
                  byte  1, -1,  1,  0,  1, -1,  1,  0      '          State 2   
                  byte  0,  1,  0,  0,  0, -1,  0, -1      '          State 3   

                  byte  1,  0,  1,  0,  1,  0,  1,  0      ' Piece 1  State 0   
                  byte  0, -1,  0,  0,  0,  1,  0,  2      '          State 1   
                  byte  1,  0,  1,  0,  1,  0,  1,  0      '          State 2   
                  byte  0, -1,  0,  0,  0,  1,  0,  2      '          State 3   

                  byte  1,  0,  0,  1,  1,  0,  0,  1      ' Piece 2  State 0   
                  byte  0, -1,  1,  0,  0,  1,  0,  1      '          State 1   
                  byte  0, -1,  1,  0,  0, -1,  1,  0      '          State 2   
                  byte  0, -1,  0,  0,  0,  1,  0,  1      '          State 3   

                  byte  1,  0,  0,  1,  1,  0,  0,  1      ' Piece 3  State 0   
                  byte  0, -1,  1,  0,  1,  1,  1,  1      '          State 1   
                  byte  1,  0,  0,  1,  1,  0,  0,  1      '          State 2   
                  byte  0, -1,  1,  0,  1,  1,  1,  1      '          State 3   

                  byte  0,  0,  1,  1,  0,  0,  1,  1      ' Piece 4  State 0   
                  byte  1, -1,  1,  0,  0,  1,  0,  1      '          State 1   
                  byte  0,  0,  1,  1,  0,  0,  1,  1      '          State 2   
                  byte  1, -1,  1,  0,  0,  1,  0,  1      '          State 3   

                  byte  0,  0,  0,  1,  0,  0,  0,  1      ' Piece 5  State 0   
                  byte  0,  0,  0,  1,  0,  0,  0,  1      '          State 1   
                  byte  0,  0,  0,  1,  0,  0,  0,  1      '          State 2   
                  byte  0,  0,  0,  1,  0,  0,  0,  1      '          State 3     

                  byte  1,  0,  1,  1,  1,  0,  1,  1      ' Piece 6  State 0    
                  byte  1, -1,  0,  0,  0,  1,  0,  1      '          State 1    
                  byte  1,  0, -1, -1,  1,  0, -1, -1      '          State 2    
                  byte  0, -1,  0,  0,  0,  1,  0,  1      '          State 3    


tabDown           byte -1,  1,  0,  0,  1,  0,  1,  0      ' Piece 0  State 0    
                  byte  1,  1,  0,  1,  1,  1,  0,  1      '          State 1    
                  byte  1,  0,  0,  0, -1,  0, -1,  0      '          State 2    
                  byte  0,  1, -1, -1,  0,  1, -1, -1      '          State 3    

                  byte -2,  0, -1,  0,  0,  0,  1,  0      ' Piece 1  State 0    
                  byte  0,  2,  0,  2,  0,  2,  0,  2      '          State 1    
                  byte -2,  0, -1,  0,  0,  0,  1,  0      '          State 2    
                  byte  0,  2,  0,  2,  0,  2,  0,  2      '          State 3    

                  byte -1,  0,  0,  1,  1,  0,  1,  0      ' Piece 2  State 0    
                  byte  0,  1,  1,  0,  0,  1,  1,  0      '          State 1    
                  byte  1,  0,  0,  0, -1,  0, -1,  0      '          State 2    
                  byte  0,  1, -1,  0,  0,  1, -1,  0      '          State 3    

                  byte -1,  1,  0,  1,  1,  0,  1,  0      ' Piece 3  State 0    
                  byte  1,  1,  0,  0,  1,  1,  0,  0      '          State 1    
                  byte -1,  1,  0,  1,  1,  0,  1,  0      '          State 2    
                  byte  1,  1,  0,  0,  1,  1,  0,  0      '          State 3    

                  byte -1,  0,  0,  1,  1,  1,  1,  1      ' Piece 4  State 0    
                  byte  0,  1,  1,  0,  0,  1,  1,  0      '          State 1    
                  byte -1,  0,  0,  1,  1,  1,  1,  1      '          State 2    
                  byte  0,  1,  1,  0,  0,  1,  1,  0      '          State 3    

                  byte -1,  1,  0,  1,  0,  1,  0,  1      ' Piece 5  State 0    
                  byte -1,  1,  0,  1,  0,  1,  0,  1      '          State 1    
                  byte -1,  1,  0,  1,  0,  1,  0,  1      '          State 2    
                  byte -1,  1,  0,  1,  0,  1,  0,  1      '          State 3    

                  byte -1,  0,  0,  0,  1,  1,  1,  1      ' Piece 6  State 0    
                  byte  0,  1,  1, -1,  0,  1,  1, -1      '          State 1    
                  byte -1,  0,  0,  0,  1,  0,  1,  0      '          State 2    
                  byte -1,  1,  0,  1, -1,  1,  0,  1      '          State 3    


strWS             byte "PROPTRIS v1.3 NE ws 1/2020                            http://propeller.ws-nbg.de", 0

strYour           byte "Your level:   ", 0
strFull           byte "Full lines:   ", 0
strScore          byte "Score", 0
strHelp           byte "H E L P", 0
strPause          byte "F1 :Pause", 0
strReboot         byte "F12:Reboot", 0
strLeft           byte $0B, "  :Left", 0
strRight          byte $0C, "  :Right", 0
strRotate         byte $0D, "  :Rotate", 0
strDrop           byte $0E, "  :Drop", 0
strDraw           byte "1  :Draw Next", 0
strSpeed          byte "2  :Speed Up", 0

strClr            byte "             ", 0

strNext           byte "Next:", 0
strPlay           byte "Play PROPTRIS!", 0
strStat           byte "STATISTICS", 0
strField1         byte $0F, "          ", $0F, 0
strField2         byte $0F, $0F, $0F, $0F, $0F, $0F, $0F, $0F, $0F, $0F, $0F, $0F, 0
strDot            byte ". . . . .", 0
strSt             byte "-   0", 0
strSum1           byte "------------", 0
strSum2           byte "Sum    :   0", 0
strClear          byte "          ", 0
strOver1          byte "            ", 0
strOver2          byte " GAME  OVER ", 0
strOver3          byte "R  :Restart", 0

strPlay2          byte "  Play PROPTRIS !  ", $0D, $0A, 0
strSpace2         byte "                   ", $0D, $0A, 0
strOnce           byte "Once more? (Y/N) >  ", 0

strSetup1         byte "PROPTRIS  Setup", 0
strSetup2         byte "P L A Y", 0
strSetup3         byte "D E M O", 0
strSetup4         byte "Show   NEXT", 0
strSetup5         byte "Hide   NEXT", 0
strSetup6         byte "P H A N T O M", 0
strSetup7         byte "N O R M A L", 0
strSetup8         byte "Selected level", 0
strSetup9         byte "Start height", 0
strSetup10        byte "0      4     10     14", 0
strSetup11a       byte "Press ", 0
strSetup11b       byte "SPACE", 0
strSetup11c       byte " to Start, ", 0
strSetup11d       byte "F12", 0
strSetup11e       byte " to Reboot.", 0


strPath           byte "C:\PROPTRIS\>", 0
                  byte "C:\>", 0

strPathWork       byte "C:\WORK\>", 0

strVolume         byte "Volume in Drive C: PROPTRIS", $0D, $0A, $0D, $0A, 0

strDir            byte ".              <DIR>   01/09/2020  01:30:00", $0D, $0A       
                  byte "..             <DIR>   01/09/2020  01:30:00", $0D, $0A
                  byte "PROPTRIS EXE   72.180  01/09/2020  01:30:00", $0D, $0A, $0D, $0A, 0

                  byte "PROPTRIS       <DIR>   01/09/2020  01:30:00", $0D, $0A
                  byte "PROPTRIS BAT       26  01/09/2020  01:30:00", $0D, $0A, $0D, $0A, 0

strCmd1           byte "CLS", 0
strCmd2           byte "PROPTRIS", 0
strCmd3           byte "EXIT", 0
strCmd4           byte "CD..", 0
strCmd5           byte "CD ..", 0
strCmd6           byte "CD\", 0
strCmd7           byte "CD \", 0
strCmd8           byte "CD PROPTRIS", 0
strCmd9           byte "CD \PROPTRIS", 0
strCmd10          byte "DIR", 0

'**************************************************************************************************************************************

fontbmp  

file  "font.bmp"


'**************************************************************************************************************************************

DAT

tone0             long $000F_070C, $000F_070C, $000E_070C, $000E_070C, $000D_070C, $000D_070C, $000C_070C, $000C_070C
                  long $000B_070C, $000B_070C, $000A_070C, $000A_070C, $0009_070C, $0009_070C, $0008_070C, $0000_0000
                  long $FFFFFFFF

tone1             long $000C_0400, $000C_0400, $0000_0000
                  long $FFFFFFFF

tone2             long $000F_0300, $000F_02E0, $000F_02C0, $000F_02A0, $000F_0280, $0000_0000
                  long $FFFFFFFF

tone3             long $000F_0100, $000F_0110, $000F_0120, $000F_0130, $000F_0140, $000F_0150, $0000_0000 
                  long $FFFFFFFF

tone4             long $000F_0100, $000F_0200, $000F_0100, $000F_0200, $000F_0100, $0000_0000 
                  long $FFFFFFFF

tone5             long $000C_0500, $000C_0500, $000C_0500, $000D_0500, $000C_0500, $000B_0500, $0000_0000
                  long $FFFFFFFF

tone6             long $000F_0100, $000F_0108, $000F_0110, $000F_0118, $000F_0120, $000F_0128, $000F_0130, $000F_0138
                  long $000F_0140, $000F_0148, $000F_0150, $000F_0158, $000F_0160, $000F_0168, $000F_0170, $000F_0178
                  long $000F_0180, $000F_0188, $000F_0190, $000F_0198, $000F_01A0, $000F_01A0, $000F_01A0, $0000_0000
                  long $FFFFFFFF

noise0            long $10_00_04_0F
                  long $FFFFFFFF

noise1            long $10_00_38_1F
                  long $FFFFFFFF



music             long $0357011D, $FC000000, $00000D0A
                  long $0357011C, $FC000000, $00000D0C
                  long $0357011C, $FC000000, $00000D0C
                  long $0357011D, $FC000000, $00000C0C
                  long $0357011D, $FC000000, $00000A0C
                  long $0357011C, $FC000000, $0000070C
                  long $0357011C, $FC000000, $0000060C
                  long $0357011D, $FC000000, $0000050C
                  long $0357011D, $FC000000, $0000040B
                  long $0357011C, $FC000000, $0000030B
                  long $0357011C, $FC000000, $0000020B
                  long $0357011D, $FE000000, $0000000B
                  long $0357011D, $FA0001AC, $000D000B
                  long $0357011C, $FA000153, $000D000B
                  long $0357011C, $FA00023B, $000D000B
                  long $0357011D, $FA0001AC, $000C000B
                  long $0357011D, $FA000153, $000A000A
                  long $0357011C, $FA00023B, $0007000A
                  long $0357011C, $FA0001AC, $0006000A
                  long $0357011D, $FA000153, $0005000A
                  long $0357011D, $FA00023B, $0004000A
                  long $0357011C, $FA0001AC, $0003000A
                  long $0357011C, $FA000153, $0002000A
                  long $0357011D, $FE00023B, $0000000A
                  long $0475011D, $FC0001AC, $00000D09
                  long $0475011C, $FC000153, $00000D09
                  long $0475011C, $FC00023B, $00000D09
                  long $0475011D, $FC0001AC, $00000C09
                  long $0475011D, $FC000153, $00000A09
                  long $0475011C, $FC00023B, $00000709
                  long $0475011C, $FC0001AC, $00000609
                  long $0475011D, $FC000153, $00000509
                  long $0475011D, $FC00023B, $00000409
                  long $0475011C, $FC0001AC, $00000309
                  long $0475011C, $FC000153, $00000209
                  long $0475011D, $FE00023B, $00000009
                  long $0475011D, $FB0001AC, $000B0000
                  long $0475011D, $FB000152, $000D0000
                  long $0475011D, $FB0001FB, $000D0000
                  long $0475011D, $FB0001AC, $000D0000
                  long $0475011D, $FB000153, $000D0000
                  long $0475011D, $FB0001FB, $000D0000
                  long $0475011D, $FB0001AB, $000D0000
                  long $0475011D, $FB000153, $000D0000
                  long $0475011D, $FB0001FC, $000C0000
                  long $0475011D, $FB0001AB, $000C0000
                  long $0475011D, $FB000152, $000C0000
                  long $0475011D, $FB0001FC, $000C0000
                  long $0357011D, $F90001AC, $000D0D00
                  long $0357011D, $F9000153, $000D0D00
                  long $0357011D, $F90001FC, $000D0D00
                  long $0357011D, $F90001AC, $000D0C00
                  long $0357011D, $F9000153, $000C0A00
                  long $0357011D, $F90001FC, $000C0700
                  long $0357011D, $F90001AC, $000C0600
                  long $0357011D, $F9000153, $000C0500
                  long $0357011D, $F90001FC, $000B0400
                  long $0357011D, $F90001AC, $000B0300
                  long $0357011D, $F9000153, $000B0200
                  long $0357011D, $FB0001FC, $000B0000
                  long $0357011D, $FB000153, $000B0000
                  long $0357011D, $FB0000FD, $000D0000
                  long $0357011D, $FB0001DF, $000D0000
                  long $0357011D, $FB000153, $000D0000
                  long $0357011D, $FB0000FE, $000D0000
                  long $0357011D, $FB0001DF, $000D0000
                  long $0357011D, $FB000152, $000D0000
                  long $0357011D, $FB0000FE, $000D0000
                  long $0357011D, $FB0001E0, $000C0000
                  long $0357011D, $FB000152, $000C0000
                  long $0357011D, $FB0000FD, $000C0000
                  long $0357011D, $FB0001E0, $000C0000
                  long $0327011D, $F9000153, $000C0D00
                  long $0327011D, $F90000FD, $000C0D00
                  long $0327011D, $F90001DF, $000C0D00
                  long $0327011D, $F9000153, $000C0C00
                  long $0327011D, $F90000FE, $000B0A00
                  long $0327011D, $F90001DF, $000B0700
                  long $0327011D, $F9000152, $000B0600
                  long $0327011D, $F90000FE, $000B0500
                  long $0327011D, $F90001E0, $000B0400
                  long $0327011D, $F9000152, $000B0300
                  long $0327011D, $F90000FD, $000B0200
                  long $0327011D, $FB0001E0, $000B0000
                  long $0327011D, $FB000153, $000D0000
                  long $0327011D, $FB00010D, $000D0000
                  long $0327011D, $FB0001E0, $000D0000
                  long $0327011D, $FB000153, $000C0000
                  long $0327011D, $FB00010D, $000A0000
                  long $0327011D, $FB0001E0, $00070000
                  long $0327011D, $FB000153, $00060000
                  long $0327011D, $FB00010D, $00050000
                  long $0327011D, $FB0001E0, $00040000
                  long $0327011D, $FB000153, $00030000
                  long $0327011D, $FB00010D, $00020000
                  long $0327011D, $FF0001E0, $00000000
                  long $02FA011D, $FC000153, $00000D0A
                  long $02FA011C, $FC00010D, $00000D0C
                  long $02FA011C, $FC0001E0, $00000D0C
                  long $02FA011D, $FC000153, $00000C0C
                  long $02FA011D, $FC00010D, $00000A0C
                  long $02FA011C, $FC0001E0, $0000070C
                  long $02FA011C, $FC000153, $0000060C
                  long $02FA011D, $FC00010D, $0000050C
                  long $02FA011D, $FC0001E0, $0000040B
                  long $02FA011C, $FC000153, $0000030B
                  long $02FA011C, $FC00010D, $0000020B
                  long $02FA011D, $FE0001E0, $0000000B
                  long $0140011D, $F80001C5, $000D0D0B
                  long $0140011C, $F80001C5, $000D0D0B
                  long $0140011C, $F80001C5, $000D0D0B
                  long $0140011D, $F80001C5, $000C0C0B
                  long $0140011D, $F80001C5, $000A0A0A
                  long $0140011C, $F80001C5, $0007070A
                  long $0140011C, $F80001C5, $0006060A
                  long $0140011D, $F80001C5, $0005050A
                  long $0140011D, $F80001C5, $0004040A
                  long $0140011C, $F80001C5, $0003030A
                  long $0140011C, $F80001C5, $0002020A
                  long $0140011D, $FE0001C5, $0000000A
                  long $0475011D, $FC0001C5, $00000D09
                  long $0475011C, $FC0001C5, $00000D09
                  long $0475011C, $FC0001C5, $00000D09
                  long $0475011D, $FC0001C5, $00000C09
                  long $0475011D, $FC0001C5, $00000A09
                  long $0475011C, $FC0001C5, $00000709
                  long $0475011C, $FC0001C5, $00000609
                  long $0475011D, $FC0001C5, $00000509
                  long $0475011D, $FC0001C5, $00000409
                  long $0475011C, $FC0001C5, $00000309
                  long $0475011C, $FC0001C5, $00000209
                  long $0475011D, $FE0001C5, $00000009
                  long $0475011D, $FB0001C5, $000B0000
                  long $0475011D, $FB00017C, $000D0000
                  long $0475011D, $FB000280, $000D0000
                  long $0475011D, $FB0001C5, $000D0000
                  long $0475011D, $FB00017D, $000D0000
                  long $0475011D, $FB000280, $000D0000
                  long $0475011D, $FB0001C4, $000D0000
                  long $0475011D, $FB00017D, $000D0000
                  long $0475011D, $FB000281, $000C0000
                  long $0475011D, $FB0001C4, $000C0000
                  long $0475011D, $FB00017C, $000C0000
                  long $0475011D, $FB000281, $000C0000
                  long $02FA011D, $F90001C5, $000D0D00
                  long $02FA011D, $F900017D, $000D0D00
                  long $02FA011D, $F9000281, $000D0D00
                  long $02FA011D, $F90001C5, $000D0C00
                  long $02FA011D, $F900017D, $000C0A00
                  long $02FA011D, $F9000281, $000C0700
                  long $02FA011D, $F90001C5, $000C0600
                  long $02FA011D, $F900017D, $000C0500
                  long $02FA011D, $F9000281, $000B0400
                  long $02FA011D, $F90001C5, $000B0300
                  long $02FA011D, $F900017D, $000B0200
                  long $02FA011D, $FB000281, $000B0000
                  long $02FA011D, $FB000153, $000B0000
                  long $02FA011D, $FB0000FD, $000D0000
                  long $02FA011D, $FB0001DF, $000D0000
                  long $02FA011D, $FB000153, $000D0000
                  long $02FA011D, $FB0000FE, $000D0000
                  long $02FA011D, $FB0001DF, $000D0000
                  long $02FA011D, $FB000152, $000D0000
                  long $02FA011D, $FB0000FE, $000D0000
                  long $02FA011D, $FB0001E0, $000C0000
                  long $02FA011D, $FB000152, $000C0000
                  long $02FA011D, $FB0000FD, $000C0000
                  long $02FA011D, $FB0001E0, $000C0000
                  long $0327011D, $F9000153, $000C0D00
                  long $0327011D, $F90000FD, $000C0D00
                  long $0327011D, $F90001DF, $000C0D00
                  long $0327011D, $F9000153, $000C0C00
                  long $0327011D, $F90000FE, $000B0A00
                  long $0327011D, $F90001DF, $000B0700
                  long $0327011D, $F9000152, $000B0600
                  long $0327011D, $F90000FE, $000B0500
                  long $0327011D, $F90001E0, $000B0400
                  long $0327011D, $F9000152, $000B0300
                  long $0327011D, $F90000FD, $000B0200
                  long $0327011D, $FB0001E0, $000B0000
                  long $0327011D, $FB000153, $000D0000
                  long $0327011D, $FB00010D, $000D0000
                  long $0327011D, $FB0001E0, $000D0000
                  long $0327011D, $FB000153, $000C0000
                  long $0327011D, $FB00010D, $000A0000
                  long $0327011D, $FB0001E0, $00070000
                  long $0327011D, $FB000153, $00060000
                  long $0327011D, $FB00010D, $00050000
                  long $0327011D, $FB0001E0, $00040000
                  long $0327011D, $FB000153, $00030000
                  long $0327011D, $FB00010D, $00020000

                  long $FFFFFFFF

{{

Permission is hereby granted, free of charge, to any person obtaining a copy of this
software and associated documentation files (the "Software"), to deal in the Software
without restriction, including without limitation the rights to use, copy, modify,
merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to the following
conditions:

The above copyright notice and this permission notice shall be included in all copies
or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

}}

                       '